// This file is made available under Elastic License 2.0.
// This file is based on code available under the Apache license here:
//   https://github.com/apache/incubator-doris/blob/master/be/test/olap/del_vector_test.cpp

// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#include "storage/del_vector.h"

#include <gtest/gtest.h>

namespace starrocks {

// NOLINTNEXTLINE
TEST(DelVector, testLoadSave) {
    DelVector dv;
    dv.set_empty();
    ASSERT_TRUE(dv.empty());
    std::shared_ptr<DelVector> ndv;
    std::vector<uint32_t> dels = {1, 3, 5, 7, 90000};
    dv.add_dels_as_new_version(dels, 1, &ndv);
    ASSERT_FALSE(ndv->empty());
    std::string raw = ndv->save();
    DelVector dv2;
    ASSERT_TRUE(dv2.load(1, raw.data(), raw.size()).ok());
    ASSERT_GT(dv2.memory_usage(), 0);
    ASSERT_EQ(dv2.cardinality(), dels.size());
};

} // namespace starrocks
